@@ -1,7 +1,7 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 from trytond.model import ModelView, ModelSQL, fields
-from trytond.wizard import Wizard
+from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond.backend import FIELDS
 from trytond.pyson import Eval, PYSONEncoder, Date, Get
 from trytond.transaction import Transaction
@@ -14,11 +14,13 @@ class Line(ModelSQL, ModelView):
     _description = __doc__
 
     employee = fields.Many2One('company.employee', 'Employee', required=True,
-        select=1, domain=[('company', '=', Get(Eval('context', {}), 'company'))])
-    date = fields.Date('Date', required=True, select=1)
+        select=True, domain=[
+            ('company', '=', Get(Eval('context', {}), 'company')),
+            ])
+    date = fields.Date('Date', required=True, select=True)
     hours = fields.Float('Hours', digits=(16, 2), required=True)
     work = fields.Many2One('timesheet.work', 'Work',
-            required=True, select=1, domain=[
+            required=True, select=True, domain=[
                 ('timesheet_available', '=', True),
             ])
     description = fields.Char('Description')
@@ -32,8 +34,6 @@ class Line(ModelSQL, ModelView):
 
     def default_employee(self):
         user_obj = Pool().get('res.user')
-        employee_obj = Pool().get('company.employee')
-
         employee_id = None
         if Transaction().context.get('employee'):
             employee_id = Transaction().context['employee']
@@ -43,7 +43,6 @@ class Line(ModelSQL, ModelView):
                 employee_id = user.employee.id
         if employee_id:
             return employee_id
-        return False
 
     def default_date(self):
         date_obj = Pool().get('ir.date')
@@ -60,9 +59,9 @@ class Line(ModelSQL, ModelView):
 Line()
 
 
-class EnterLinesInit(ModelView):
-    'Enter Lines Init'
-    _name = 'timesheet.enter_lines.init'
+class EnterLinesStart(ModelView):
+    'Enter Lines'
+    _name = 'timesheet.line.enter.start'
     _description = __doc__
     employee = fields.Many2One('company.employee', 'Employee', required=True,
             domain=[('company', '=', Get(Eval('context', {}), 'company'))])
@@ -76,55 +75,36 @@ class EnterLinesInit(ModelView):
         line_obj = Pool().get('timesheet.line')
         return line_obj.default_date()
 
-EnterLinesInit()
+EnterLinesStart()
 
 
 class EnterLines(Wizard):
     'Enter Lines'
-    _name = 'timesheet.enter_lines'
-    states = {
-        'init': {
-            'result': {
-                'type': 'form',
-                'object': 'timesheet.enter_lines.init',
-                'state': [
-                    ('end', 'Cancel', 'tryton-cancel'),
-                    ('enter', 'Enter', 'tryton-ok', True),
-                ],
-            },
-        },
-        'enter': {
-            'result': {
-                'type': 'action',
-                'action': '_action_enter_lines',
-                'state': 'end',
-            },
-        }
-    }
+    _name = 'timesheet.line.enter'
 
-    def _action_enter_lines(self, data):
-        pool = Pool()
-        model_data_obj = pool.get('ir.model.data')
-        act_window_obj = pool.get('ir.action.act_window')
-        employee_obj = pool.get('company.employee')
-        act_window_id = model_data_obj.get_id('timesheet', 'act_line_form')
-        res = act_window_obj.read(act_window_id)
-        date = data['form']['date']
-        date = Date(date.year, date.month, date.day)
-        res['pyson_domain'] = PYSONEncoder().encode([
-            ('employee', '=', data['form']['employee']),
-            ('date', '=', date),
+    start = StateView('timesheet.line.enter.start',
+        'timesheet.line_enter_start_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Enter', 'enter', 'tryton-ok', default=True),
             ])
-        res['pyson_context'] = PYSONEncoder().encode({
-            'employee': data['form']['employee'],
-            'date': date,
-            })
+    enter = StateAction('timesheet.act_line_form')
 
-        if data['form']['employee']:
-            employee = employee_obj.browse(data['form']['employee'])
-            res['name'] += " - " + employee.rec_name
+    def do_enter(self, session, action):
+        date = session.start.date
+        date = Date(date.year, date.month, date.day)
+        action['pyson_domain'] = PYSONEncoder().encode([
+                ('employee', '=', session.start.employee.id),
+                ('date', '=', date),
+                ])
+        action['pyson_context'] = PYSONEncoder().encode({
+                'employee': session.start.employee.id,
+                'date': date,
+                })
+        action['name'] += " - " + session.start.employee.rec_name
+        return action, {}
 
-        return res
+    def transition_enter(self, session):
+        return 'end'
 
 EnterLines()
 
@@ -134,7 +114,7 @@ class HoursEmployee(ModelSQL, ModelView):
     _name = 'timesheet.hours_employee'
     _description = __doc__
 
-    employee = fields.Many2One('company.employee', 'Employee', select=1)
+    employee = fields.Many2One('company.employee', 'Employee', select=True)
     hours = fields.Float('Hours', digits=(16, 2))
 
     def table_query(self):
@@ -161,50 +141,36 @@ class HoursEmployee(ModelSQL, ModelView):
 HoursEmployee()
 
 
-class OpenHoursEmployeeInit(ModelView):
-    'Open Hours Employee Init'
-    _name = 'timesheet.open_hours_employee.init'
+class OpenHoursEmployeeStart(ModelView):
+    'Open Hours per Employee'
+    _name = 'timesheet.hours_employee.open.start'
     _description = __doc__
     start_date = fields.Date('Start Date')
     end_date = fields.Date('End Date')
 
-OpenHoursEmployeeInit()
+OpenHoursEmployeeStart()
 
 
 class OpenHoursEmployee(Wizard):
     'Open Hours per Employee'
-    _name = 'timesheet.open_hours_employee'
-    states = {
-        'init': {
-            'result': {
-                'type': 'form',
-                'object': 'timesheet.open_hours_employee.init',
-                'state': [
-                    ('end', 'Cancel', 'tryton-cancel'),
-                    ('open', 'Open', 'tryton-ok', True),
-                ],
-            },
-        },
-        'open': {
-            'result': {
-                'type': 'action',
-                'action': '_action_open',
-                'state': 'end',
-            },
-        },
-    }
+    _name = 'timesheet.hours_employee.open'
 
-    def _action_open(self, data):
-        model_data_obj = Pool().get('ir.model.data')
-        act_window_obj = Pool().get('ir.action.act_window')
-        act_window_id = model_data_obj.get_id('timesheet',
-            'act_hours_employee_form')
-        res = act_window_obj.read(act_window_id)
-        res['pyson_context'] = PYSONEncoder().encode({
-            'start_date': data['form']['start_date'],
-            'end_date': data['form']['end_date'],
-            })
-        return res
+    start = StateView('timesheet.hours_employee.open.start',
+        'timesheet.hours_employee_open_start_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Open', 'open_', 'tryton-ok', default=True),
+            ])
+    open_ = StateAction('timesheet.act_hours_employee_form')
+
+    def do_open_(self, session, action):
+        action['pyson_context'] = PYSONEncoder().encode({
+                'start_date': session.start.start_date,
+                'end_date': session.start.end_date,
+                })
+        return action, {}
+
+    def transition_open_(self, session):
+        return 'end'
 
 OpenHoursEmployee()
 
@@ -214,10 +180,10 @@ class HoursEmployeeWeekly(ModelSQL, ModelView):
     _name = 'timesheet.hours_employee_weekly'
     _description = __doc__
 
-    year = fields.Char('Year', select=1)
-    week = fields.Integer('Week', select=1)
-    employee = fields.Many2One('company.employee', 'Employee', select=1)
-    hours = fields.Float('Hours', digits=(16, 2), select=1)
+    year = fields.Char('Year', select=True)
+    week = fields.Integer('Week', select=True)
+    employee = fields.Many2One('company.employee', 'Employee', select=True)
+    hours = fields.Float('Hours', digits=(16, 2), select=True)
 
     def __init__(self):
         super(HoursEmployeeWeekly, self).__init__()
@@ -252,10 +218,10 @@ class HoursEmployeeMonthly(ModelSQL, ModelView):
     _name = 'timesheet.hours_employee_monthly'
     _description = __doc__
 
-    year = fields.Char('Year', select=1)
-    month = fields.Integer('Month', select=1)
-    employee = fields.Many2One('company.employee', 'Employee', select=1)
-    hours = fields.Float('Hours', digits=(16, 2), select=1)
+    year = fields.Char('Year', select=True)
+    month = fields.Integer('Month', select=True)
+    employee = fields.Many2One('company.employee', 'Employee', select=True)
+    hours = fields.Float('Hours', digits=(16, 2), select=True)
 
     def __init__(self):
         super(HoursEmployeeMonthly, self).__init__()
