@@ -1,9 +1,11 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 from trytond.model import ModelView, ModelSQL, fields
-from trytond.wizard import Wizard
-from trytond.pyson import PYSONEncoder
+from trytond.wizard import Wizard, StateView, StateAction, Button
+from trytond.pyson import PYSONEncoder, Not, Bool, Eval
 from trytond.transaction import Transaction
+from trytond.pool import Pool
+
 
 class Work(ModelSQL, ModelView):
     'Work'
@@ -13,15 +15,26 @@ class Work(ModelSQL, ModelView):
     name = fields.Char('Name', required=True)
     active = fields.Boolean('Active')
     parent = fields.Many2One('timesheet.work', 'Parent', left="left",
-            right="right", select=2, ondelete="RESTRICT")
-    left = fields.Integer('Left', required=True, select=1)
-    right = fields.Integer('Right', required=True, select=1)
+            right="right", select=True, ondelete="RESTRICT")
+    left = fields.Integer('Left', required=True, select=True)
+    right = fields.Integer('Right', required=True, select=True)
     children = fields.One2Many('timesheet.work', 'parent', 'Children')
     hours = fields.Function(fields.Float('Timesheet Hours', digits=(16, 2),
         help="Total time spent on this work"), 'get_hours')
     timesheet_available = fields.Boolean('Available on timesheets',
-            help="Allow to fill in timesheets with this work")
-    company = fields.Many2One('company.company', 'Company', required=True)
+        states={
+            'readonly': Bool(Eval('timesheet_lines', [0])),
+            },
+        help="Allow to fill in timesheets with this work")
+    company = fields.Many2One('company.company', 'Company', required=True,
+        select=True)
+    timesheet_lines = fields.One2Many('timesheet.line', 'work',
+        'Timesheet Lines',
+        depends=['timesheet_available', 'active'],
+        states={
+            'invisible': Not(Bool(Eval('timesheet_available'))),
+            'readonly': Not(Bool(Eval('active'))),
+            })
 
     def __init__(self):
         super(Work, self).__init__()
@@ -38,11 +51,17 @@ class Work(ModelSQL, ModelView):
     def default_active(self):
         return True
 
+    def default_left(self):
+        return 0
+
+    def default_right(self):
+        return 0
+
     def default_timesheet_available(self):
         return True
 
     def default_company(self):
-        return Transaction().context.get('company') or False
+        return Transaction().context.get('company')
 
     def check_parent_company(self, ids):
         for work in self.browse(ids):
@@ -87,7 +106,7 @@ class Work(ModelSQL, ModelView):
 
         Transaction().cursor.execute(clause, all_ids + args)
 
-        hours_by_wt = dict((i[0], i[1]) for i in 
+        hours_by_wt = dict((i[0], i[1]) for i in
             Transaction().cursor.fetchall())
         to_compute = dict.fromkeys(all_ids, True)
         works = self.browse(all_ids)
@@ -102,6 +121,7 @@ class Work(ModelSQL, ModelView):
         if not ids:
             return {}
         res = {}
+
         def _name(work):
             if work.parent:
                 return _name(work.parent) + '\\' + work.name
@@ -111,6 +131,14 @@ class Work(ModelSQL, ModelView):
             res[work.id] = _name(work)
         return res
 
+    def copy(self, ids, default=None):
+        if default is None:
+            default = {}
+        default = default.copy()
+        if 'timesheet_lines' not in default:
+            default['timesheet_lines'] = None
+        return super(Work, self).copy(ids, default=default)
+
     def write(self, ids, vals):
         child_ids = None
         if not vals.get('active', True):
@@ -127,48 +155,36 @@ class Work(ModelSQL, ModelView):
 Work()
 
 
-class OpenWorkInit(ModelView):
-    'Open Work Init'
-    _name = 'timesheet.work.open.init'
+class OpenWorkStart(ModelView):
+    'Open Work'
+    _name = 'timesheet.work.open.start'
     _description = __doc__
     from_date = fields.Date('From Date')
     to_date = fields.Date('To Date')
-OpenWorkInit()
+
+OpenWorkStart()
 
 
 class OpenWork(Wizard):
     'Open Work'
     _name = 'timesheet.work.open'
-    states = {
-        'init': {
-            'result': {
-                'type': 'form',
-                'object': 'timesheet.work.open.init',
-                'state': [
-                    ('end', 'Cancel', 'tryton-cancel'),
-                    ('open', 'Open', 'tryton-ok', True),
-                ],
-            },
-        },
-        'open': {
-            'result': {
-                'type': 'action',
-                'action': '_action_open_work',
-                'state': 'end',
-            },
-        },
-    }
 
-    def _action_open_work(self, data):
-        model_data_obj = self.pool.get('ir.model.data')
-        act_window_obj = self.pool.get('ir.action.act_window')
-        act_window_id = model_data_obj.get_id('timesheet', 'act_work_tree2')
-        res = act_window_obj.read(act_window_id)
-        res['pyson_context'] = PYSONEncoder().encode({
-            'from_date': data['form']['from_date'],
-            'to_date': data['form']['to_date'],
-            })
-        return res
+    start = StateView('timesheet.work.open.start',
+        'timesheet.work_open_start_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Open', 'open_', 'tryton-ok', default=True),
+            ])
+    open_ = StateAction('timesheet.act_work_hours_board')
+
+    def do_open_(self, session, action):
+        action['pyson_context'] = PYSONEncoder().encode({
+                'from_date': session.start.from_date,
+                'to_date': session.start.to_date,
+                })
+        return action, {}
+
+    def transition_open_(self, session):
+        return 'end'
 
 OpenWork()
 
@@ -176,42 +192,23 @@ OpenWork()
 class OpenWork2(OpenWork):
     _name = 'timesheet.work.open2'
 
-    def _action_open_work(self, data):
-        model_data_obj = self.pool.get('ir.model.data')
-        act_window_obj = self.pool.get('ir.action.act_window')
-        act_window_id = model_data_obj.get_id('timesheet', 'act_work_form2')
-        res = act_window_obj.read(act_window_id)
-        res['pyson_context'] = PYSONEncoder().encode({
-            'from_date': data['form']['from_date'],
-            'to_date': data['form']['to_date'],
-            })
-        return res
+    open_ = StateAction('timesheet.act_work_form2')
 
 OpenWork2()
 
 
 class OpenWorkGraph(Wizard):
     _name = 'timesheet.work.open_graph'
-    states = {
-        'init': {
-            'result': {
-                'type': 'action',
-                'action': '_action_open_work',
-                'state': 'end',
-            },
-        },
-    }
+    start_state = 'open_'
+    open_ = StateAction('timesheet.act_work_form3')
 
-    def _action_open_work(self, data):
-        model_data_obj = self.pool.get('ir.model.data')
-        act_window_obj = self.pool.get('ir.action.act_window')
-        work_obj = self.pool.get('timesheet.work')
+    def do_open_(self, session, action):
+        pool = Pool()
+        work_obj = pool.get('timesheet.work')
 
-        act_window_id = model_data_obj.get_id('timesheet', 'act_work_form3')
-        res = act_window_obj.read(act_window_id)
         if 'active_id' in Transaction().context:
             work = work_obj.browse(Transaction().context['active_id'])
-            res['name'] = res['name'] + ' - ' + work.rec_name
-        return res
+            action['name'] = action['name'] + ' - ' + work.rec_name
+        return action, {}
 
 OpenWorkGraph()
